/**
 * 环境检测对象, 用于在应用初始化时提供简便的环境查询对象 ,
 *
 * sdk configuration , 初始化入口,
 * 可以传入cordova或者weixin ,api 对象 [选填] , 检测当前环境是否和传入的对象相符
 *
 * Created by tonyjiang on 16/2/24.
 */


class EnvClass {

    constructor(win , options) {

        this.constant = {
            os : {
                mobile : "mobile",
                ios : "ios",
                android : "android",
                computer : "computer"
            },

            browser : {
                ie : 'internet_explorer',
                chrome : 'chrome',
                firefox : 'firefox',
                safari : 'safari',
                opera : 'opera',
                unknown : 'unknown'
            },

            platform : {
                weixin : 'weixin',
                chanjet : 'chanjet',
                browser : 'browser'
            }
        };

        this._init(win);
    }

    _init(win){
        this._env_variables = {};

        if(!window || undefined == window){
            throw new Error('env-check must run on browser environment');
            return false;
        }

        this._window = win || window ;



        let _ua = this._window.navigator.userAgent.toLowerCase();

        this._checkBrowser(_ua);
        this._checkOs(_ua);
        this._checkPlatform(_ua);

        Object.freeze(this._env_variables);

    }

    /**
     * 检测浏览器 , ie , firefox , chrome , opera , safari , unknown;
     * @param ua
     * @private
     */
    _checkBrowser (ua){

        if(ua.indexOf("msie") != -1){
            this._env_variables.browser = this.constant.browser.ie;
        }else if(ua.indexOf("firefox") != -1){
            this._env_variables.browser = this.constant.browser.firefox;
        }else if(ua.indexOf("chrome") != -1){
            this._env_variables.browser = this.constant.browser.chrome;
        }else if(ua.indexOf("opera") != -1){
            this._env_variables.browser = this.constant.browser.opera;
        }else if(ua.indexOf("safari") != -1){
            this._env_variables.browser = this.constant.browser.safari;
        }else{
            this._env_variables.browser = this.constant.browser.unknown;
        }

    }

    /**
     * 检测操作系统 , iphone , android , mobile , computer;
     * @param ua
     * @private
     */
    _checkOs (ua){

        if(ua.indexOf('iphone') != -1){
            this._env_variables.os = this.constant.os.ios;
        }else if(ua.indexOf('android') != -1){
            this._env_variables.os = this.constant.os.android;
        }else if(ua.indexOf('mobile') != -1){
            this._env_variables.os = this.constant.os.mobile;
        }else{
            this._env_variables.os = this.constant.os.computer;
        }

    }

    /**
     * 检测平台, weixin , dingding , native ... so on;
     * @param ua
     * @private
     */
    _checkPlatform(ua){
        if(ua.indexOf('micromessenger') != -1){
            this._env_variables.platform = this.constant.platform.weixin;
        }else if(ua.indexOf('chanjet') != -1){
            this._env_variables.platform = this.constant.platform.chanjet;


        }else{
            this._env_variables.platform = this.constant.platform.browser;
        }
    }
    
    get env(){
        return this._env_variables;
    }

    /**
     * 获取当前平台
     * @returns {String}
     */
    get platform(){
        return this._env_variables.platform;
    }

    /**
     * 获取当前操作系统
     * @returns {String}
     */
    get os(){
        return this._env_variables.os;
    }

    /**
     * 获取当前浏览器
     * @returns {String}
     */
    get browser(){
        return this._env_variables.browser;
    }

    set window(win){
        this._init(win);
    }

    getInstance(){
        return this;
    }

}

let _instance = new EnvClass(window);

module.exports = _instance;